-- File Name   : tb_lfsr1_1.vhd
--  Comments   : Test bench for the random number generation
--				 Version 1_1
------------------------------------------------------------------
library ieee;
    use ieee.std_logic_1164.all;
    use ieee.std_logic_textio.all;
	use ieee.numeric_std.ALL;
    use std.textio.all;
	use work.lfsr_pkg.all;
    
entity tb_lfsr1_1 is
end entity;

architecture test of tb_lfsr1_1 is

    constant PERIOD  : time   := 20 ns;
    constant log_file: string := "res.log";

    signal clk       : std_logic := '0';
    signal reset     : std_logic := '1';
    signal enable    : std_logic := '0';
    signal count     : std_logic_vector (LFSR_W-1 downto 0);
    signal endSim	 : boolean   := false;

    component lfsr1 is
    port (
        reset     : in  std_logic;                       
        clk       : in  std_logic;               
        enable    : in  std_logic;               
        count     : out std_logic_vector (LFSR_W-1 downto 0)   
    );
    end component;
    

begin
    clk     <= not clk after PERIOD/2;
    reset   <= '0' after  PERIOD*10;
    enable  <= '1' after  PERIOD*12;

	-- End the simulation
	process 
	begin
		if (endSim) then
			assert false 
				report "End of simulation." 
				severity failure; 
		end if;
		wait until (clk = '1');
	end process;	

    lfrs1_inst : lfsr1
    port map (
        clk      => clk,
        reset    => reset,
		enable   => enable,
        count    => count
    );

	-- Save data to file
    process 
		file 		file_id: 	text;
		variable 	line_num: 	line;
		variable	cnt:		integer := 0;
	begin
		-- Open the file
		file_open(file_id, log_file, WRITE_MODE);
		wait until (enable = '1');
		wait until (clk = '1');
		
		-- Loop and write all values to a file		
		for cnt in 0 to 2047 loop
			write(line_num, to_integer(unsigned(count)) ); 
			writeline(file_id, line_num);
			wait until (clk = '1');
		end loop;
		
		file_close(file_id);
		endSim <= true;
		wait until (clk = '1');

	end process;	
end architecture;